// DemoVCDlg.cpp : implementation file
//

#include "stdafx.h"
#include "DemoVC.h"
#include "DemoVCDlg.h"
#include "uCalcCPP.cpp"
#include  <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

double W, H;
void CartesianLineTo(CDC* pDC, double X, double Y);

// uCalc callback routines

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
DWORD _stdcall MyErrorHandler(long t)
{
   //uc_char represents wchar_t in Unicode and char in Multi-byte
   uc_char ErrLocation[20];
   static long AlreadyDisplayedOnce = 0;   

   // uc_GetSymbol and uc_GetLocation work only for parse-time errors
   // (such as Syntax Error or Undefined Identifier)
   if(AlreadyDisplayedOnce == ucFalse)   
      MessageBox(0, "Error Handler message: " + ucErrorMessage(0, t) + '\n'
         + "Offending symbol: " + ucErrSymbol(t) + '\n'
         + "Error Location: " + uc_LongToString(ucErrLocation(t), ErrLocation, 10) + '\n' + '\n'
         + "This message box will not be displayed for the next error. " + '\n'
         + "Remove 'AlreadyDisplayedOnce = 1;' in the demo to change this. ",
		 _T("uCalc"), MB_OK);
   
   // Remove the line below if you want the message box to be
   // displayed every time there's an error.
   AlreadyDisplayedOnce = 1;
   return ucAbort;
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
void _stdcall MyArea(DWORD Expr)
{
   long double MyLength, MyWidth;   

   MyLength = ucParam(Expr, 1);
   MyWidth = ucParam(Expr, 2);   
   
   if(MyLength < 0) ucRaiseErrorMessage(Expr, "Length cannot be negative");
   if(MyWidth  < 0) ucRaiseErrorMessage(Expr, "Width cannot be negative");

   ucReturn(Expr, MyLength * MyWidth);
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
long double _stdcall NonNativeFunc(double a, long& b, byte c)
{
   return a + b + c;
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
void _stdcall MyIIF_Numeric(DWORD Expr)
{
   double Condition;
   DWORD TruePart, FalsePart;

   Condition = ucParam(Expr, 1);
   TruePart = ucParamHandle(Expr, 2);
   FalsePart = ucParamHandle(Expr, 3);

   if (Condition != 0)
   ucReturn(Expr, ucEvaluate(TruePart));
   else ucReturn(Expr, ucEvaluate(FalsePart));
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
void _stdcall MyIIF_String(DWORD Expr)
{
   double Condition;
   DWORD TruePart, FalsePart;

   Condition = ucParam(Expr, 1);
   TruePart = ucParamHandle(Expr, 2);
   FalsePart = ucParamHandle(Expr, 3);

   if (Condition != 0)
   ucReturnStr(Expr, ucEvaluateStr(TruePart));
   else ucReturnStr(Expr, ucEvaluateStr(FalsePart));
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
void _stdcall Native_MsgBox(DWORD Expr)
{
   // The 2nd and 3rd args are swapped so that MsgBox is uniform with the demo for VB and other compilers
   ucReturn(Expr, MessageBox(0, ucParamStr(Expr, 1), ucParamStr(Expr, 3), ucParam(Expr, 2)));
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
long _stdcall MyMsgBox(long Prompt, long Buttons, long Title)
{
   return MessageBox(0, ucEvaluateStr(Prompt), ucEvaluateStr(Title), Buttons);
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
long _stdcall nn_MsgBox(CString& Prompt, long Buttons, CString& Title)
{
   return MessageBox(0, Prompt, Title, Buttons);
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
void _stdcall MyAverage(DWORD Expr)
{
   long x;
   double Total = 0;
   
   for(x = 1; x <= ucParamCount(Expr); x++)
      Total = Total + ucParam(Expr, x);

   ucReturn(Expr, Total / ucParamCount(Expr));
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
void _stdcall MyLeft(DWORD Expr)
{
   CString MyString;
   MyString = ucParamStr(Expr, 1);

   ucReturnStr(Expr, MyString.Left(ucParam(Expr, 2)));
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
void _stdcall StringRepeat(DWORD Expr)
{   
   CString MyString, TotalString;
   long x, Count;

   MyString = ucParamStr(Expr, 1);
   Count = ucParam(Expr, 2);

   for(x = 1; x <= Count; x++)
   TotalString = TotalString + MyString;
   
   ucReturnStr(Expr, TotalString);
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
void _stdcall ucSum(DWORD Expr)
{
   DWORD Expression, VarHandle;
   double Start, Finish, sStep, x, Total;

   Expression = ucParamHandle(Expr, 1);
   Start = ucParam(Expr, 2);
   Finish = ucParam(Expr, 3);
   sStep = ucParam(Expr, 4);
   VarHandle = ucParamHandle(Expr, 5);

   x = Start;
   Total = 0;

   while(x <= Finish)
   {
      ucSetVariableValue(VarHandle, x);
      Total = Total + ucEvaluate(Expression);
      x = x + sStep;
   }

   ucReturn(Expr, Total);
}

// IMPORTANT: Remember to use _stdcall for your uCalc callbacks
void _stdcall ucSolve(DWORD Expr)
{
   DWORD Expression, Variable;
   long Iterations = 0;
   double a, b, fa, fb, Value, tmp;

   Expression = ucParamHandle(Expr, 1);
   a = ucParam(Expr, 2);
   b = ucParam(Expr, 3);
   Variable = ucParamHandle(Expr, 4);

   ucSetVariableValue(Variable, a); fa = ucEvaluate(Expression);
   ucSetVariableValue(Variable, b); fb = ucEvaluate(Expression);
   
   if(fb < fa) {tmp = a; a = b; b = tmp;}  // swap a, b
    
   while(fabs(b - a) > 0.000000000000001)
   {
      ucSetVariableValue(Variable, (a + b) / 2);
      
      Value = ucEvaluate(Expression);
      
      if(Value == 0) {a = (a + b) / 2; break;}

      if(Value < 0) a = (a + b) / 2; else b = (a + b) / 2;

      Iterations = Iterations + 1;
      if(Iterations == 100) break;
   }
   
   if(fabs(Value) > 0.0000000001) ucRaiseErrorMessage(Expr, "Solution not found");

   ucReturn(Expr, a);
}

void Test(CString Expr, CString Answer, long t = 0)
{
   if(Answer != ucEvalStr(Expr, t)) MessageBox(0, Expr + '\n' + ucEvalStr(Expr, t) + '\n' + Answer, "", 0);
}

void TestExpand(CString Expr, CString Answer, long t = 0)
{
   if(Answer != ucExpand(Expr, t)) MessageBox(0, Expr + '\n' + ucExpand(Expr, t) + '\n' + Answer, "", 0);
}


/////////////////////////////////////////////////////////////////////////////
// CDemoVCDlg dialog

CDemoVCDlg::CDemoVCDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDemoVCDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDemoVCDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	// The definitions here are for "callback" routines.
	// The actual code for these routines can be found elsewhere in this file.
	// Do a search for MyErrorHandler, for instance, to see the actual code for it.
	// Each line ends with the name of a routine.  FuncPtr converts the
	// address pointer of a routine to 32-bit integer for use with uCalc().
   
	// The MyErrorHandler() function will be called whenever and wherever an
	// error (such as "Syntax Error") is raised.
	uCalc(uc_AddErrorHandler, "", FuncPtr(MyErrorHandler));

	// Native callback definitions, which start with "Native: " are recommended for
	// speed, and for routines that use uCalc's default numeric and string types.
	// MyArea() returns the product of multiplying the two arguments.
	// MyLeft() returns the left-most characters of a string.  The second argument represents the number of characters.
	// MessgeBox() displays a message box.  The 1st argument is required.  The others are optional.   
	ucDefineFunction("Native: MyArea(Length, Width)", MyArea);
	ucDefineFunction("Native: MyLeft(Text As String, Count) As String", MyLeft);
	ucDefineFunction("Native: MsgBox(Prompt As String, Buttons = 0, Title As String = 'uCalc')", Native_MsgBox);

	// The three consecutive dots "..." mean that MyAverage can take any number of arguments
	// (however, because "x" is specified, there must be at least one argument).
	ucDefineFunction("Native: MyAverage(x ...)", MyAverage);

	// Two versions of MyIIF can co-exist peacefully with the same name because
	// they are defined with different argument types (numeric vs string).
	// By passing the last two arguments ByExpr, the callback can chose to evaluate only one of of them.
	ucDefineFunction("Native: MyIIf(ByVal cond, ByExpr TruePart, ByExpr FalsePart)", MyIIF_Numeric);
	ucDefineFunction("Native: MyIIf(cond, ByExpr TruePart As String, ByExpr FalsePart As String) As String", MyIIF_String);

	// The following defines the "*" operator so that MyString * n returns MyString repeated n times.
	// For instance, "He " * 3, would return "He He He".  20 represents the precedence level.  It is
	// arbitrarily set at the same level as that of the "*" multiplication operator defined in uCalcCPP.cpp.
	ucDefineOperator("Native: 20 {MyString As String} * {Number} As String", StringRepeat);

	// Non-native callbacks are intended for routines that were not created with
	// uCalc in mind, such as the Windows API, or other pre-existing DLL routines.
	// It is convenient (no need to adapt the routines; all you need is a function
	// address), but not as fast as Native callbacks.
	// NonNativeFunc() simply adds the three arguments.
	// nn_MsgBox is a Non-Native version of MessageBox, which displays a message box.
	ucDefineFunction("NonNativeFunc(ByVal a As Double, ByRef b As Long, ByVal c As Byte) As Extended", NonNativeFunc);
	ucDefineFunction("nn_MsgBox(ByRef Prompt As String, ByVal Buttons As Long = 0, ByRef Title As String = 'uCalc') As Long", nn_MsgBox);

	// This definition is for a summation.  See the callback Sum() routine for the
	// actual code, which runs a loop that adds up the total for the expression in
	// the first argument a number of times based on the second and third arguments.
	// For instance Sum(g^2+1, 1, 5, 1, g) returns 60 and Sum(x^2, 1, 10) returns 385.
	// The last two arguments are optional, so they default to 1 and x if omitted.
	//
	// The first argument is passed "ByExpr".  So instead of being evaluated before
	// being passed to the callback the way an ordinary argument would be, a handle
	// for the expression is passed so that the callback can evaluate it (in this
	// case numerous times).
	//
	// The last argument is passed "ByHandle".  This causes the callback to receive
	// a handle for the variable being passed, in such a way that it can be linked to
	// the summation counter, and integrated into the expression in the first argument.
	//
	// The actual function being defined is Sum_().  Then a syntax construct named
	// Sum() is defined in such a way that the last argument gets defined as a local
	// variable.  So if you evaluate Sum(x^2, 5, 10, 1, x), the local "x" in this
	// expression will not interfere with a pre-existing variable named x.  Also you
	// do not need to declare a variable ahead of time to use it as a counter for Sum().
	ucDefineFunction("Native: Sum_(ByExpr Expr, Start, Finish, Step, ByHandle Var)", ucSum);
	ucDefineSyntax("Sum({Expr}, {Start}, {Finish} [, {Step=1} [, {Var=x}]])"
			"::= Local({Var}, Sum_({Expr}, {Start}, {Finish}, {Step}, {Var}))");

	// The following routine solves an equation.
	// The callback code is based on the Bisection Method algorithm.
	// The concept here is very similar to that of Sum().
	// Two syntax constructs are defined.  The second one rearranges the equation
	// if it includes an equal sign.
	// For instance Solve(x^2 = 9+x) becomes Solve(x^2 - (9+x))
	// Solve(x^2 + 1 = 26) returns 5.
	// Solve(x^2 + 1 = 26, -1000, 0) returns -5.
	ucDefineFunction("Native: Solve_(ByExpr Expr, a, b, ByHandle Var)", ucSolve);
	ucDefineSyntax("Solve({Expr} [, {a=-100000000} [, {b=100000000} [, {Var=x}]]]) "
		"::= Local({Var}, Solve_({Expr}, {a}, {b}, {Var}))");
	ucDefineSyntax("Solve({Left} = {Right} [, {etc}]) ::= Solve({Left}-({Right}) {etc: , {etc}})");
}

void CartesianLineTo(CDC* pDC, double X, double Y)
{
  int x, y;
  CRect rect;  

  x = (long)(W * (X + 10) / 20);
  y = (long)(H * (2 - Y) / 4);

  if (X == -10 || y > H || y < 0)
  pDC->MoveTo((long)x, (long)min(max(y, 0), H));
  else pDC->LineTo(x, y);  
}

void CDemoVCDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDemoVCDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CDemoVCDlg, CDialog)
	//{{AFX_MSG_MAP(CDemoVCDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_btnEval, OnbtnEval)
	ON_BN_CLICKED(IDC_chkExprEvalExamples, OnchkExprEvalExamples)
	ON_BN_CLICKED(IDC_chkDefExamples, OnchkDefExamples)
	ON_BN_CLICKED(IDC_btnDefine, OnbtnDefine)
	ON_BN_CLICKED(IDC_btnSum, OnbtnSum)
	ON_BN_CLICKED(IDC_btnPlot, OnbtnPlot)
	ON_EN_CHANGE(IDC_txtExpression, OnChangetxtExpression)
	ON_EN_CHANGE(IDC_txtDefinition, OnChangetxtDefinition)
	ON_BN_CLICKED(IDC_chkInvalidOp, OnchkInvalidOp)
	ON_BN_CLICKED(IDC_chkDenormalOp, OnchkDenormalOp)
	ON_BN_CLICKED(IDC_chkDivisionBy0, OnchkDivisionBy0)
	ON_BN_CLICKED(IDC_chkOverflow, OnchkOverflow)
	ON_BN_CLICKED(IDC_chkUnderflow, OnchkUnderflow)
	ON_BN_CLICKED(IDC_chkPrecisionLoss, OnchkPrecisionLoss)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDemoVCDlg message handlers

BOOL CDemoVCDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetDlgItemText(IDC_txtExpression, _T("3+4/5-8"));
	SetDlgItemText(IDC_txtSumMax, _T("100000"));
	SetDlgItemText(IDC_txtSumExpression, _T("x^2+5*x-10"));
	SetDlgItemText(IDC_txtPlotEq, _T("-sin(x)/2"));
	SetDlgItemText(IDC_txtDefinition, _T("Var: MyVariable = 1234.5"));
        CheckDlgButton(IDC_chkInvalidOp, BST_CHECKED);
        CheckDlgButton(IDC_chkDenormalOp, BST_CHECKED);
        CheckDlgButton(IDC_chkDivisionBy0, BST_CHECKED);
        CheckDlgButton(IDC_chkOverflow, BST_CHECKED);
        CheckDlgButton(IDC_chkUnderflow, BST_CHECKED);
        CheckDlgButton(IDC_chkPrecisionLoss, BST_CHECKED);	

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CDemoVCDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CDemoVCDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CDemoVCDlg::OnbtnEval() 
{
	CString MyText;

	// ucEval() evaluates an expression and returns a numeric result.
	// ucEvalStr() can evaluate expressions of any data type (numeric, string, or other).
	// If the result is numeric ucEvalStr conveniently converts it to a string for you.
	GetDlgItemText(IDC_txtExpression, MyText);
	SetDlgItemText(IDC_txtResult, ucEvalStr(MyText));	
}

void CDemoVCDlg::OnchkExprEvalExamples() 
{
    ShellExecute(0, _T("open"), _T("DemoExpr.Txt"), _T(""), _T(""), SW_NORMAL);
}

void CDemoVCDlg::OnchkDefExamples() 
{
    ShellExecute(0, _T("open"), _T("DemoDefs.Txt"), _T(""), _T(""), SW_NORMAL);
}

void CDemoVCDlg::OnbtnDefine() 
{
	CString MyText;

	GetDlgItemText(IDC_txtDefinition, MyText);

	// ucDefine can define a variable, function, operator, syntax construct,
	// constant, pattern, data type, and more.
	//
	// Examples:
	//
	// ucDefine("Var: MyVariable As String");
	// ucDefine("Func: Area(Length, Width) = Length * Width");
	// ucDefine("Op: 50 {x}%  :=  x / 100");   // 50 is an arbitrary precedence level.
	// ucDefine("Syntax: {a} + {a} ::= 'Two {a}s are better than one {a}'");
	// ucDefine("Const: Pi = Atan(1) * 4");
	ucDefine(MyText);	
}

void CDemoVCDlg::OnbtnSum() 
{
   CString txtSumMax, txtSumExpression, txtSumTotal;
   CString txtElapsedTime;
   char txtElapsedTime_[25], txtSumTotal_[25];
   long TimerStart, TimerFinish;
   double x = 1, SumTotal = 0, SumMax;
   DWORD ExprHandle, xHandle;   

   GetDlgItemText(IDC_txtSumMax, txtSumMax);
   GetDlgItemText(IDC_txtSumExpression, txtSumExpression);
   SumMax = uc_StringToLong(txtSumMax);
   
   xHandle = ucDefineVariable("x", &x);
   ExprHandle = ucParse(txtSumExpression);
   TimerStart = GetTickCount();

   while (x <= SumMax)
   {
      SumTotal = SumTotal + ucEvaluate(ExprHandle);
      x++;
   }
 
   TimerFinish = GetTickCount();
   gcvt(SumTotal, 15, txtSumTotal_);
   gcvt((float)(TimerFinish-TimerStart)/(float)1000, 15, txtElapsedTime_);

   txtSumTotal = txtSumTotal_;
   txtElapsedTime = txtElapsedTime_;
   
   SetDlgItemText(IDC_txtSumResult, txtSumTotal);
   SetDlgItemText(IDC_lblElapsed, "Elapsed time: "+txtElapsedTime+" seconds");

   ucReleaseItem(ExprHandle);
   ucReleaseItem(xHandle);	
}

void CDemoVCDlg::OnbtnPlot() 
{   
   CString txtPlotEq;
   double x = -10;
   DWORD EqHandle, xHandle;
   CRect rect;

   CWnd* h = GetDlgItem(IDC_PICTURE1);
   h->GetClientRect(rect);
   CDC* pDC = h->GetDC();

   h->GetClientRect(rect);
   pDC->FillRect(rect, CBrush::FromHandle((HBRUSH)GetStockObject(LTGRAY_BRUSH)));
  
   W = rect.Width();
   H = rect.Height();

   pDC->MoveTo(0, (long)H/2);


   GetDlgItemText(IDC_txtPlotEq, txtPlotEq);
   
   xHandle = ucDefineVariable("x", &x);
   EqHandle = ucParse(txtPlotEq);  

   while (x <= 10)
   {
      CartesianLineTo(pDC, x, ucEvaluate(EqHandle));
      x = x + 0.125;
   }

   ucReleaseItem(EqHandle);
   ucReleaseItem(xHandle);	
}

void CDemoVCDlg::OnChangetxtExpression() 
{
	CString txtExpression;

	GetDlgItemText(IDC_txtExpression, txtExpression);
   	SetDlgItemText(IDC_lblEvalSource1, '\x22'+txtExpression+'\x22');	
}

void CDemoVCDlg::OnChangetxtDefinition() 
{
	CString txtDefinition;

	GetDlgItemText(IDC_txtDefinition, txtDefinition);
   	SetDlgItemText(IDC_lblDefineSource, '\x22'+txtDefinition+'\x22');	
}

void CDemoVCDlg::OnchkInvalidOp() 
{
   // If toggled, then an expression such as 0 / 0 raises an error.
   // If toggled again, then 0 / 0 returns NaN.
   uCalc(uc_ToggleFPU, "", uc_FPU_Mask_InvalidOp);	
}

void CDemoVCDlg::OnchkDenormalOp() 
{
   uCalc(uc_ToggleFPU, "", uc_FPU_Mask_DenormalOp);	
}

void CDemoVCDlg::OnchkDivisionBy0() 
{
   // If toggled, then 1 / 0 raises an error.
   // If toggled again, then 1 / 0 returns Inf.
   uCalc(uc_ToggleFPU, "", uc_FPU_Mask_ZeroDivide);	
}

void CDemoVCDlg::OnchkOverflow() 
{
   // If toggled, then 100! ^ 150 raises an error.
   // If toggled again, then 100! ^ 150 returns Inf.
   uCalc(uc_ToggleFPU, "", uc_FPU_Mask_Overflow);	
}

void CDemoVCDlg::OnchkUnderflow() 
{
   // If toggled, then (1/100!)^150 raises an error.
   // If toggled again, then (1/100!)^150 returns 0.
   uCalc(uc_ToggleFPU, "", uc_FPU_Mask_Underflow);	
}

void CDemoVCDlg::OnchkPrecisionLoss() 
{
   // If toggled, an expression that cannot be represented in exact form
   // such as "1/3" will raise an error.
   // If toggled again, "1/3" returns the closest extended precision approximation.
   uCalc(uc_ToggleFPU, "", uc_FPU_Mask_PrecisionLoss);	
}
